@@ -6,6 +6,7 @@
 #define QXCBBACKINGSTORE_H
 
 #include <qpa/qplatformbackingstore.h>
+#include <QtCore/QStack>
 
 #include <xcb/xcb.h>
 
@@ -21,21 +22,27 @@ public:
     QXcbBackingStore(QWindow *widget);
     ~QXcbBackingStore();
 
-    QPaintDevice *paintDevice();
-    void flush(QWindow *window, const QRegion &region, const QPoint &offset);
+    QPaintDevice *paintDevice() override;
+    void flush(QWindow *window, const QRegion &region, const QPoint &offset) override;
 #ifndef QT_NO_OPENGL
     void composeAndFlush(QWindow *window, const QRegion &region, const QPoint &offset,
                          QPlatformTextureList *textures, QOpenGLContext *context,
-                         bool translucentBackground);
+                         bool translucentBackground) override;
 #endif
-    QImage toImage() const;
-    void resize(const QSize &size, const QRegion &staticContents);
-    bool scroll(const QRegion &area, int dx, int dy);
+    QImage toImage() const override;
 
-    void beginPaint(const QRegion &);
+    QPlatformGraphicsBuffer *graphicsBuffer() const override;
+
+    void resize(const QSize &size, const QRegion &staticContents) override;
+    bool scroll(const QRegion &area, int dx, int dy) override;
+
+    void beginPaint(const QRegion &) override;
+    void endPaint() override;
 
 private:
     QXcbShmImage *m_image;
+    QStack<QRegion> m_paintRegions;
+    QImage m_rgbImage;
 };
 
 QT_END_NAMESPACE
